#!/bin/sh
# ============================================================================
#
# This file is part of the 'FFP-stick for ZyXEL NSA-nnn'
# http://zyxel.nas-central.org/wiki/FFP-stick
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Author: Mijzelf <Mijzelf@live.com>
#
# ============================================================================
# 
# This script is supposed to be started by usb_key_func.sh, or by itself
#
# Run without params for a brief help

SCRIPTVERSION="20151012"
LOGFILE="ffpboot.log"
WHILEBOOTING="while_booting.sh"
AFTERBOOTING="after_booting.sh"
TMPSTICK="/tmp/.ffpstick"
TMPROOT="/tmp/ffproot"

FATFILES="${WHILEBOOTING} \
		usb_key_func.sh \
		nsa210_check_file md5sum \
		nsa220_check_file \
		NSA221_check_file salted_md5sum.Zy_Private \
		nsa310_check_file_C0 salted_md5sum.libzy.so \
		STG100_check_file STG211_check_file STG212_check_file \
		nas5xx_check_file salted_md5sum.libzy.so.5xx \
		"
EXT3FILES="${AFTERBOOTING} rootfs.tgz fun_plug.tgz"

# ======================
# Filled in by commandline params
USBDEVICE=
NASTYPE=
FIRMWARE=
USELOGFILE=""
PARNERKEY_MP=""
READONLY=no
# ======================

# ===================================================
# function SplitPath
# split path in first parameter in path and filename
SPLITTEDPATH="" # Output path
SPLITTEDFILE="" # Output file
# ===================================================
SplitPath()
{
    SPLITTEDPATH=""
    SPLITTEDFILE=""
    IFSBACKUP=${IFS}; 	IFS=/
    for chunk in $1
    do
        newpath=""
        if [ "${SPLITTEDPATH}" = "" ] ; then
	    newpath="${chunk}"
	else
	    newpath="${SPLITTEDPATH}/${chunk}"
	fi
	if [ "${newpath}" = "$1" ] ; then
	    SPLITTEDFILE="${chunk}"
	elif [ "/${newpath}" = "$1" ] ; then
	    SPLITTEDPATH="/${SPLITTEDPATH}"
	    SPLITTEDFILE="${chunk}"
	else
	    SPLITTEDPATH="${newpath}"
	fi
    done
    IFS=${IFSBACKUP}
    if [ "" = "${SPLITTEDPATH}" ] || [ "." = "${SPLITTEDPATH}" ]; then
	SPLITTEDPATH=` pwd `
    fi
}

# ===================================================
# function FindInDatabase
# Find first argument in list. Return next item
# Arguments: $1 search item
#            rest is database
# Output
FOUNDENTRY=""
# ===================================================
FindInDatabase()
{
    if [ "$1" = "" ] ; then
	return 2
    fi
    
    SEARCHFOR=$1
    FOUNDENTRY=""
    
    shift
    while [ "$1" != "" ] ; 
    do
	if [ "$1" = "${SEARCHFOR}" ] ; then
	    FOUNDENTRY=$2
	    return 0
	fi
	shift ; shift
    done

    return 1
}

# ===================================================
# function GetNasType
# Try to find out the NAS type, and FW
NASTYPE="" # output NAS type
FIRMWARE="" # output FW
# ===================================================
GetNasType()
{
    models="D401 NSA210 \
	D501 NSA220 \
	DC01 NSA221 \
	DA01 NSA310_TDC \
	A203 NSA310_ZyXEL \
	DD01 NSA320 \
	AA03 NSA325 \
	A403 STG100 \
	A803 STG211 \
	AB03 STG212 \
	AE03 NSA325v2 \
	AD03 NSA310s \
	AF03 NSA320s \
	B103 NAS540 \
	B203 NAS520 \
	"

    echo Try to determine NAS type...

    if [ "" = "${NASTYPE}" ] ; then
    	for file in /zyxel/mnt/info/modelid /firmware/mnt/info/modelid
    	do
    	    if [ -r $file ] ; then
    	    	model=` cat $file `
		echo -n "according to ${file}: ${model}->"
        	if FindInDatabase ${model} ${models} ;
		then
	    	    NASTYPE=${FOUNDENTRY}
	            echo ${NASTYPE}
	            break
	        else
	            echo Unknown
	        fi
	    fi
	done
    fi

    if [ "" = "${NASTYPE}" ] && [ -x /zyxel/sbin/mrd_model ]; then
	model=` /zyxel/sbin/mrd_model -p `
	echo -n "according to /zyxel/sbin/mrd_model -p: ${model}->"
        if FindInDatabase ${model} ${models} ;
	then
	    NASTYPE=${FOUNDENTRY}
	    echo ${NASTYPE}
	else
	    echo Unknown
	fi
    fi

    if [ "" = "${NASTYPE}" ] ; then
	kernel=` uname -r `
	echo -n "according to uname -r: ${kernel}->"
	case ${kernel} in
	    2.6.18.6*)
		NASTYPE=NSA220
		;;
	    2.6.24.4*)
		NASTYPE=NSA210
		;;
	    *)
		NASTYPE=Unknown-${model}
		;;
	esac
	echo ${NASTYPE}
    fi

    for version in /zyxel/mnt/info/fwversion /etc/fwversion /firmware/mnt/info/fwversion
    do
	    if [ -r $version ] ; then
		FIRMWARE=` cat $version `
		break
	    fi
    done

    if [ "" = "${FIRMWARE}" ]; then
	FIRMWARE="Undetermined"
    fi

    FIRMWARE=` echo $FIRMWARE | tr -d '"' | tr -d "'" `
    NASTYPE=` echo $NASTYPE | tr -d '"' | tr -d "'" ` 

    echo type ${NASTYPE}, fw ${FIRMWARE}
}

# =======================================
# function JumpToRepartition
# copies all needed files to ramdisk,
# and passes execution to script there
# =======================================
JumpToRepartition()
{
    echo Copy files to ramdisk...
    
    mkdir -p ${TMPSTICK}

    filelist=${FATFILES}
    local copied=""
    local notcopied=""
    for loop in 1 2 3
    do
	for file in ${filelist}
	do
	    if [ -r ${SPLITTEDPATH}/${file} ] ; then
		if cp ${SPLITTEDPATH}/${file} ${TMPSTICK}/${file}; then
		    copied="${copied} ${file}"
		else
		    rm ${TMPSTICK}/${file}
		    notcopied="${notcopied} ${file}"
		fi
	    fi
	done
	if [ 1 -eq ${loop} ]; then
	    filelist=${EXT3FILES}
	else
	    filelist=${SPLITTEDFILE}
	fi
    done

    echo "Copied${copied}"
    if [ "" != "${notcopied}" ]; then
	echo "*Not* copied${notcopied}"
    fi

    if [ "" != "${USELOGFILE}" ]; then
	exec >/dev/null 2>&1
	cp -a ${SPLITTEDPATH}/${USELOGFILE} ${TMPSTICK}/${USELOGFILE}
        exec >>${TMPSTICK}/${USELOGFILE} 2>&1
        sync
    fi

    exec ${TMPSTICK}/${SPLITTEDFILE} Repartition -m ${PARNERKEY_MP} -d ${USBDEVICE} -l -n ${NASTYPE}
}

# =========================================
# function FirstCall
# This is the original call of the script
# =========================================
FirstCall()
{
    echo ffpstick started at ` date `
    GetNasType
    
    PARNERKEY_MP="/mnt/parnerkey"
    if [ "${NASTYPE}" = "NSA220" ]; then
	if cat ${SPLITTEDPATH}/nsa220_check_file | grep -v parnerkey >/dev/null ; then
		PARNERKEY_MP="/mnt"
	fi
    fi

# =======================================
# Added 20151012
# So far all fw 5.x boxes have a diverging mountpoint
# and infodir. So let's use one to detect the other
# =======================================
    if [ -f /firmware/mnt/info/modelid ] ; then
# =======================================
# Added 20150903
# FFP won't run on a 64k kernel
# =======================================
	if [ "64" = "$( cat /proc/self/smaps | grep PageSize | head -n 1 | awk '{ print $2 }' )" ] ; then
	    echo FATAL can\'t run on a NSA5xx with 64kB kernel. You need firmware 5.10 or newer
	    exit 1
	fi
        PARNERKEY_MP="/mnt/partnerkey"
    fi

    if [ "${PARNERKEY_MP}" != "${SPLITTEDPATH}" ]; then
	echo FATAL Wrong mountpoint ${SPLITTEDPATH}, should have been ${PARNERKEY_MP}
	echo Cowardly refusing to install/start FFP in an unknown environment
	exit 1
    fi

    if [ -e /tmp/${SPLITTEDFILE} ]; then
	echo For some reason this script is called multiple times
	exit 1
    fi

    echo -ne Find the current usb device...
    usbdev=`cat /proc/mounts | grep ${SPLITTEDPATH} | grep -v ram | tail -n 1 | awk '{print $1}'`
    if [ "${usbdev}" = "" ]; then
	echo FATAL Usbdevice not found
	exit 1
    fi
    echo -ne usb device is ${usbdev}
    USBDEVICE=` echo ${usbdev} | sed 's/[0-9]*//g' `
    if [ "${USBDEVICE}" = "${usbdev}" ] ; then
	echo
    else
	echo " on device ${USBDEVICE}"
    fi
   
    if [ -x ${SPLITTEDPATH}/${AFTERBOOTING} ]; then
	echo -ne "Check for filesystem..."
	filesystem=`cat /proc/mounts | grep ${SPLITTEDPATH} | grep -v ram | tail -n 1 | awk '{print $3}'`
	echo Filesystem is ${filesystem}
	if [ "${filesystem}" = "vfat" ] ; then
	    if [ "${USBDEVICE}" = "${usbdev}" ]; then
		echo ${usbdev} is not partitioned. Will have to partition it
		JumpToRepartition
	    else
		echo -ne "Check ${USBDEVICE} for number of partitions..."
	        partition_num=`fdisk -l ${USBDEVICE} | grep "^"${USBDEVICE} | wc -l `
		echo ${USBDEVICE} has ${partition_num} partitions
		if [ ${partition_num} -eq 1 ] ; then
		    echo Will have to repartition
		    JumpToRepartition
		fi
	    fi
	fi
    fi

    # Execute an eventual '${WHILEBOOTING}' script
    if [ "${WHILEBOOTING}" != "" ] && [ -x ${SPLITTEDPATH}/${WHILEBOOTING} ]; then
	echo Executing ${WHILEBOOTING}...
	cd ${SPLITTEDPATH}
	${SPLITTEDPATH}/${WHILEBOOTING} ${usbdev} ${SPLITTEDPATH} ${NASTYPE} "${FIRMWARE}"
    fi

    echo -ne Copy myself to /tmp...
    mkdir -p ${TMPSTICK}
    cp -pf ${SPLITTEDPATH}/${SPLITTEDFILE} ${TMPSTICK}/${SPLITTEDFILE}

    if ! [ -x ${TMPSTICK}/${SPLITTEDFILE} ]; then
	echo FATAL Failed to copy!
	exit 1
    fi

    COMMAND="${TMPSTICK}/${SPLITTEDFILE} BackgroundPolling -d ${USBDEVICE} -n ${NASTYPE} -f \"${FIRMWARE}\" -r ${READONLY} -l"
    echo And execute...
    echo "${COMMAND} &"

    if [ "" != "${USELOGFILE}" ]; then
	cp -a ${SPLITTEDPATH}/${USELOGFILE} ${TMPSTICK}/${USELOGFILE}
	# Close logfile, maybe it will disturb the unmounting hereafter
        exec >/dev/null 2>&1
        sync
    fi
    cd /tmp
    
    ${COMMAND} &
    exit 1
}

# ========================================
# function RotateLogfile
# Arguments: directory logfile
# Rotates logfile, compresses old versions
# ========================================
RotateLogfile()
{
    logdir=$1
    logfile=$2
    logpath=${logdir}/${logfile}

    echo Rotating logfiles...

    oldlog=${logpath}.9.gz
    if [ -r ${oldlog} ] ; then
	echo rm ${oldlog}
	rm ${oldlog}
    fi
    
    for nr in 8 7 6 5 4 3 2 1
    do
	oldlog=${logpath}.${nr}.gz
	if [ -r ${oldlog} ] ; then
	    newnr=`expr $nr + 1`
	    newlog=${logpath}.${newnr}.gz
	    echo mv ${oldlog} ${newlog}
	    mv ${oldlog} ${newlog}
	fi
    done
    if [ -r ${logpath} ] ; then
	echo "cat ${logpath} | gzip -c >${oldlog}"
	cat ${logpath} | gzip -c >${oldlog}
	rm ${logpath}
    fi
}

# ==========================================================================
# function ReopenLogfile
# Arguments: device mountpoint copy/movecommand
# Mounts device on mountpoint, moves/copies current logfile to mountpoint
# and redirects output
# ==========================================================================
ReopenLogfile()
{
    if [ "" = "${USELOGFILE}" ] ; then
	return 0
    fi

    if ! [ -d $2 ]; then
        mkdir -p $2
    fi
    if ! mount $1 $2 ; then
	return 1
    fi
    $3 ${TMPSTICK}/${USELOGFILE} $2/${USELOGFILE}
    exec >>$2/${USELOGFILE} 2>&1
}

# ================================================
# function ProbeMountPoints
# Check all mountpoints for the existence of
# - usb_key_func.sh (this file) being the ffpstick
# - ffproot/after_booting.sh being the ffproot
# Outputs:
MP_FFPSTICK=""
MP_FFPROOT=""
#=================================================
ProbeMountPoints()
{
    echo Probe all mount points $1 seconds...

    mountpoints=`cat /proc/mounts | awk '{print $2}' | grep -v /mnt/sd`

    for mp in ${mountpoints}
    do
	# count number of / in string. We're only interested with 2 or more
	slashes=` echo ${mp} | tr -cd / `
	if [ ${#slashes} -gt 1 ]; then
	    echo probe ${mp}...

	    if [ "" != "${USELOGFILE}" ] && [ "" = "${MP_FFPSTICK}"  ] ; then
		if cat /proc/mounts | grep ${mp} | grep ${USBDEVICE}1 >/dev/null ; then
		    MP_FFPSTICK=${mp}
		    echo "found ffpstick on ${MP_FFPSTICK}, moving logfile"
		    if [ "${READONLY}" = "rotate" ] ; then
			RotateLogfile ${MP_FFPSTICK} ${USELOGFILE}
		    fi
		    mv ${SPLITTEDPATH}/${USELOGFILE} ${MP_FFPSTICK}/${USELOGFILE}
		    exec >>${MP_FFPSTICK}/${USELOGFILE} 2>&1
		fi
	    fi

	    if [ "" = "${MP_FFPROOT}" ] && [ -x ${mp}/ffproot/${AFTERBOOTING} ] ; then
		MP_FFPROOT=${mp}/ffproot
		echo found ffproot on ${MP_FFPROOT}
		local device=`cat /proc/mounts | grep ${mp} | awk '{print $1}'`
		if [ "${device}" = "${USBDEVICE}2" ] ; then
		    # This is the USB stick. Try to remount it relatime
			# or, if relatime is not supported, noatime
			# or, if ext3 fails, ext2
		    mount ${device} ${mp} -t ext3 -o remount,relatime 2>/dev/null || \
		    mount ${device} ${mp} -t ext3 -o remount,noatime 2>/dev/null || \
		    mount ${device} ${mp} -t ext3 -o remount,relatime 2>/dev/null || \
		    mount ${device} ${mp} -t ext3 -o remount,noatime 2>/dev/null 
		fi
	    fi
	fi

    done


    if [ "" = "${MP_FFPROOT}" ]; then
        echo No /anymountpoint/ffproot/${AFTERBOOTING} found
        return 1
    fi

    return 0
} 

##################################################
## Create ext2/3 partiton on stick, and move files back
##################################################
CreateAndMountExt3Partition()
{
	echo Create ext3 filesystem on ${USBDEVICE}2 and mount...

	if ! mke2fs -L FFPROOT -j ${USBDEVICE}2 ; then
		# NAS5x0 doesn't have an mke2fs. Use our own
		echo Cannot use the build-in mke2fs. Trying my own...
		ln -s md5sum ${TMPSTICK}/mke2fs
		if ! ${TMPSTICK}/mke2fs -L FFPROOT ${USBDEVICE}2 ; then
			FatalErrorInRepartitioning "Couldn\'t create ext3 fs on ${USBDEVICE}2"
		fi
	fi

	sync

	if ! TryToMount ${USBDEVICE}2 ${PARNERKEY_MP} ext3 continue
	then
		TryToMount ${USBDEVICE}2 ${PARNERKEY_MP} ext2 fail
	fi

	PrepareExt3Partition ${PARNERKEY_MP}

	if [ "$1" = "noumount" ] ; then
		return 0
	fi
    
	echo -ne Unmount ${USBDEVICE}2...
	umount ${PARNERKEY_MP}
	echo done
	return 0
}

# ===========================================
# function BackgroundPolling
# function waits until NAS is booted, and has
# mounted the stick partition, and then starts ffp
# ===========================================
BackgroundPolling()
{
    echo Wait for the stick to be mounted again by the firmware...
    sleep 30

    if [ -r ${SPLITTEDPATH}/${SPLITTEDFILE} ]; then
	rm ${SPLITTEDPATH}/${SPLITTEDFILE}
    fi
    
    local loopteller=0
    local waitforstick=0
    while [ 900 -gt ${loopteller} ] 
    do
        ProbeMountPoints ${loopteller} 
    	
	if [ "" != "${MP_FFPROOT}" ] ; then
	    if [ 3 -gt ${waitforstick} ] && [ "" = "${MP_FFPSTICK}" ] ; then
		waitforstick=`expr ${waitforstick} + 1`
	    else
		cd ${MP_FFPROOT}
		# telnetd -l /bin/sh
		exec ${MP_FFPROOT}/${AFTERBOOTING} ${MP_FFPROOT} ${NASTYPE} "${FIRMWARE}"
	        exit 0
    	    fi
	else
    	    if [ "" = "${MP_FFPSTICK}" ] ; then
		waitforstick=0
	    else 
    		if [ 6 -eq ${waitforstick} ] ; then
    		    # Have a look if we can mount the 2nd partition ourselfs
		    echo "60 seconds should be enough for anybody ;)"
		    echo Will try to mount ffproot myself
		    mkdir -p ${TMPROOT}
		    mount ${USBDEVICE}2 ${TMPROOT}
    		fi
    		waitforstick=`expr ${waitforstick} + 1`
    	    fi
	fi
	loopteller=`expr ${loopteller} + 10`
	sleep 10
    done

    # Have a look if we can mount the 2nd partition ourselfs
    ReopenLogile ${USBDEVICE}1 ${TMPSTICK} mv
    echo Final attempt to mount the stick myself
    mkdir -p ${TMPROOT}
    if mount ${USBDEVICE}2 ${TMPROOT} ; then
	MP_FFPROOT=${TMPROOT}/ffproot
	if [ -x ${MP_FFPROOT}/${AFTERBOOTING} ] ; then
	    cd ${MP_FFPROOT}
	    exec ${MP_FFPROOT}/${AFTERBOOTING} ${MP_FFPROOT} ${NASTYPE} "${FIRMWARE}"
	    exit 0
	fi

	echo FATAL No ${MP_FFPROOT}/${AFTERBOOTING} found 
	exit 0
    fi
	
    echo FATAL Mounting failed
    echo "FATAL Timeout & Giving up"
    ReopenLogfile ${USBDEVICE}1 ${TMPSTICK} mv
    exit 0
}

# ================================================
# function FatalErrorInRepartitioning
# Is called when repartitioning failed.
# It will try to put the logfile back on the stick
# and collect more info
# ================================================
FatalErrorInRepartitioning()
{
    echo FATAL $1
    
    # Try to get the logfile back on the stick
    if ! ReopenLogfile ${USBDEVICE}1  ${PARNERKEY_MP} mv ; then
    	ReopenLogfile ${USBDEVICE}  ${PARNERKEY_MP} mv
    fi
    for dmp in mounts filesystems partitions
    do
	echo =================
	echo cat /proc/${dmp}
	cat /proc/${dmp}
    done
    echo ================
    echo dmesg:
    dmesg
    exec >/dev/null 2>&1
    exit 1
}

TryToMount()
{
    echo -ne Mount $1 on $2 type $3...
    local counter=0
    mount $1 $2 -t $3
    while [ 0 -ne $? ];
    do
	echo
	counter=` expr ${counter} + 1 `
	if [ ${counter} -gt 5 ]; then
	    if [ "$4" = "fail" ] ; then
		FatalErrorInRepartitioning "Mount $1 on $2 failed"
	    else
		echo giving up
		return 1
	    fi
	fi
	echo -ne Couldn\'t mount $1 on $2. Will retry in 3 seconds...
	sleep 3
	mount $1 $2 -t $3
    done
    echo done
    return 0
}

# =========================================
# function PrepareExt3Partition
# Copy files to ext3 partition
# ========================================
PrepareExt3Partition()
{
    echo -ne Move files to ext3 partition...
    mkdir -p $1/ffproot
    for file in ${EXT3FILES} 
    do
	if [ -r ${TMPSTICK}/${file} ]; then
	    echo -ne ${file}...
	    mv ${TMPSTICK}/${file} $1/ffproot/
	fi
    done
    chmod 700 $1/ffproot/${AFTERBOOTING}
    echo done

    echo -ne Create dummy symlinks...
    cat >$1/.readme <<_EOF_
zyshd runs "rm \` find /mountpoint/of/stick -type l \`" 
after mounting any external device.
This removes all symlinks from the device, rendering FFP useless.
The bunch of symlinks here are added to overflow the commandline,
making rm fail with an "Argument list too long"
instead of deleting all symlinks
_EOF_

    # Generate long name (128 characters)
    local counter=0
    longname=LongName
    while [ ${counter} -le 3 ] ; do
	longname=${longname}${longname}
	counter=` expr ${counter} + 1 `
    done
	
    # Generate 512 symlinks (total filename length = 64k)
    counter=0
    while [ ${counter} -le 512 ] 
    do
	ln -s ".readme" $1/.${counter}${longname}
	counter=` expr ${counter} + 1 `
    done
    echo done
}

# =========================================
# function PrepareVfatPartition
# Copy files to vfat partition
# ========================================
PrepareVfatPartition()
{
    echo -ne Move files to vfat partition...
    for file in ${FATFILES} 
    do
	if [ -r ${TMPSTICK}/${file} ]; then
	    echo -ne ${file}...
	    mv ${TMPSTICK}/${file} $1/
	fi
    done
    echo -ne ${SPLITTEDFILE}...
    cp ${SPLITTEDPATH}/${SPLITTEDFILE} $1/
    echo done

    if [ "no" != ${READONLY} ] ; then
	USELOGFILE=${LOGFILE}
    fi

    if [ "" != "${USELOGFILE}" ]; then
	exec >/dev/null 2>&1
	mv ${TMPSTICK}/${USELOGFILE} $1/${USELOGFILE}
        exec >>$1/${USELOGFILE} 2>&1
        sync
    fi
}

# =========================================
# function Repartition
# Repartions the (mounted) USB-stick in a 
# ~8MB FAT partition,
# and the remaining space a ext3 partition
# =========================================
Repartition()
{
    echo Repartition ${USBDEVICE}
    echo -ne unmount ${PARNERKEY_MP}...
    umount ${PARNERKEY_MP}
    echo done

    # Clear dmesg, just in case we get a fatal error. 
    dmesg -c >/dev/null 2>&1

    echo Repartitioning...
    echo -e "o\nn\np\n1\n\n+16M\nn\np\n2\n\n\nt\n1\n4\nw\n" | fdisk ${USBDEVICE}
    if [ "0" != "$?" ]; then
        FatalErrorInRepartitioning "Partitioning failed"
    fi
    
    sleep 10

    echo Done. Creating and mounting filesystems...

    CreateAndMountExt3Partition

    echo -ne Create FAT filesystem on ${USBDEVICE}1 and mount...
    if ! mkdosfs -n FFPBOOT ${USBDEVICE}1 ; then
	# The Medion and NAS5x0 doesn't have a mkdosfs
	echo -ne "Couldn\'t use the build-in mkdosfs, trying my own..."
	ln -s md5sum ${TMPSTICK}/mkdosfs
	if ! ${TMPSTICK}/mkdosfs -n FFPBOOT ${USBDEVICE}1 ; then
		FatalErrorInRepartitioning "Couldn\'t create vfat fs on ${USBDEVICE}1"
	fi
    fi
    sync

    TryToMount ${USBDEVICE}1 ${PARNERKEY_MP} vfat fail

    PrepareVfatPartition ${PARNERKEY_MP}

    echo ...And calling the script on stick again...
    exec ${PARNERKEY_MP}/${SPLITTEDFILE} NextCall -l
}

OpenLogfile()
{
    if [ "" = "${LOGFILE}" ]; then
	return 0
    fi

    USELOGFILE=${LOGFILE}

    logfile=${SPLITTEDPATH}/${USELOGFILE}
    to_open=${logfile}

    if [ "$1" = "rotate" ] ; then
	to_open=${SPLITTEDPATH}/templog.log
    fi

    # Check if ${SPLITTEDPATH} is writable:
    PROBEFILE=${SPLITTEDPATH}/probefile
    if [ -r ${PROBEFILE} ] ; then
	rm ${PROBEFILE}
    fi
    readonly=false
    if [ -r ${PROBEFILE} ] ; then
	readonly=true
    else
	echo test >${PROBEFILE}
	if [ -r ${PROBEFILE} ] ; then
	    rm ${PROBEFILE}
	else
	    readonly=true
	fi
    fi

    if [ "${readonly}" = "true" ] ; then
	# Stick is read-only
	mkdir -p ${TMPSTICK}
	exec >>${TMPSTICK}/${USELOGFILE} 2>&1
	echo Stick is mounted readonly, logging in ram
	USELOGFILE=""
	READONLY=$1
	return 0
    fi

    exec >>${to_open} 2>&1

    if [ "$1" != "rotate" ] ; then
	return 0
    fi

    # Try to save older bootlogs
    RotateLogfile ${SPLITTEDPATH} ${USELOGFILE}
    logfile=${SPLITTEDPATH}/${USELOGFILE}

    exec >/dev/null 2>&1
    mv ${to_open} ${logfile}
    exec >>${logfile} 2>&1
}

PrintHelp()
{
    echo Usage: ${SPLITTEDFILE} command arguments
    echo "   command is one of:"
    echo "       FirstCall"
    echo "       NextCall"
    echo "       Repartition"
    echo "       BackgroundPolling"
    echo "       Help"
    echo "   arguments:"
    echo "       -f <firmware>"
    echo "       -n <nastype>"
    echo "       -d <usbdevice>"
    echo "       -m <mountpoint> (normally /mnt/parnerkey)"
    echo "       -r <readonly> no, yes, rotate"
    echo "       -l redirect output to logfile"
    echo "       -ll rotate logfiles and redirect output"
}

ReadParams()
{
    shift
    while [ "$1" != "" ] ; do
	case $1 in
	    -f)
		FIRMWARE=$2
		shift
		;;
	    -n)
		NASTYPE=$2
		shift
		;;
	    -d)
		USBDEVICE=$2
		shift
		;;
	    -m)
		PARNERKEY_MP=$2
		shift
		;;
	    -r)
		READONLY=$2
		shift
		;;
	    -h)
		PrintHelp
		;;
	    -l)
		OpenLogfile yes
		;;
	    -ll)
		OpenLogfile rotate
		;;
	    *)
		echo Unknown option $1
		;;
	esac
	shift
    done
}

# Entrypoint of the script
# Find filename and path
SplitPath $0
# Read the command
COMMAND=$1
# Check for 2nd call
if [ "${COMMAND}" = "FirstCall" ] ; then
    if [ -d ${TMPSTICK} ] ; then
	# For some reason the function is called twice
        exit 1
    fi
fi

# Read params
if [ $# -gt 1 ]; then
    ReadParams $@
fi

echo Script ${SPLITTEDFILE} version ${SCRIPTVERSION} running from ${SPLITTEDPATH}

case ${COMMAND} in
	FirstCall)
		FirstCall
		exit 1
		;;
	NextCall)
		FirstCall
		exit 1
		;;
	Repartition)
		Repartition
		;;
	BackgroundPolling)
		BackgroundPolling 
		;;
	Help)
		PrintHelp
		;;
	NSA221Finish)
		umount /mnt/parnerkey
		rmdir /mnt/parnerkey
		if [ -x /tmp/NSA221Finish.sh ]; then
		    exec /tmp/NSA221Finish.sh
		fi
		;;
	*)
		PrintHelp
		;;
esac


